@@ -11,16 +11,17 @@
 # permit persons to whom the Software is furnished to do so, subject to
 # the following conditions:
 #
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
 #
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 # EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
 
 """Automatically formats Python code to conform to the PEP 8 style guide.
 
@@ -33,6 +34,7 @@ documentation for more information.
 
 """
 
+from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
@@ -63,7 +65,7 @@ except NameError:
     unicode = str
 
 
-__version__ = '0.9.1'
+__version__ = '0.9.2'
 
 
 CR = '\r'
@@ -71,7 +73,7 @@ LF = '\n'
 CRLF = '\r\n'
 
 
-PYTHON_SHEBANG_REGEX = re.compile(r'^#!.*\bpython[23]?\b')
+PYTHON_SHEBANG_REGEX = re.compile(r'^#!.*\bpython[23]?\b\s*$')
 
 
 # For generating line shortening candidates.
@@ -213,16 +215,21 @@ class FixPEP8(object):
         self.fix_e703 = self.fix_e702
         self.fix_w191 = self.fix_e101
 
+        self._ws_comma_done = False
+
     def _fix_source(self, results):
         completed_lines = set()
         for result in sorted(results, key=_priority_key):
             if result['line'] in completed_lines:
                 continue
 
-            fixed_methodname = 'fix_%s' % result['id'].lower()
+            fixed_methodname = 'fix_' + result['id'].lower()
             if hasattr(self, fixed_methodname):
                 fix = getattr(self, fixed_methodname)
 
+                line_index = result['line'] - 1
+                original_line = self.source[line_index]
+
                 is_logical_fix = len(inspect.getargspec(fix).args) > 2
                 if is_logical_fix:
                     # Do not run logical fix if any lines have been modified.
@@ -237,6 +244,12 @@ class FixPEP8(object):
                 else:
                     modified_lines = fix(result)
 
+                if (
+                    modified_lines is None and
+                    self.source[line_index] == original_line
+                ):
+                    modified_lines = []
+
                 if modified_lines:
                     completed_lines.update(modified_lines)
                 elif modified_lines == []:  # Empty list means no fix
@@ -245,17 +258,19 @@ class FixPEP8(object):
                             '--->  Not fixing {f} on line {l}'.format(
                                 f=result['id'], l=result['line']),
                             file=sys.stderr)
-                else:  # We assume one-line fix when None
+                else:  # We assume one-line fix when None.
                     completed_lines.add(result['line'])
             else:
                 if self.options.verbose >= 3:
-                    print("--->  '%s' is not defined." % fixed_methodname,
-                          file=sys.stderr)
+                    print(
+                        "--->  '{0}' is not defined.".format(fixed_methodname),
+                        file=sys.stderr)
+
                     info = result['info'].strip()
-                    print('--->  %s:%s:%s:%s' % (self.filename,
-                                                 result['line'],
-                                                 result['column'],
-                                                 info),
+                    print('--->  {0}:{1}:{2}:{3}'.format(self.filename,
+                                                         result['line'],
+                                                         result['column'],
+                                                         info),
                           file=sys.stderr)
 
     def fix(self):
@@ -389,7 +404,16 @@ class FixPEP8(object):
     def fix_e122(self, result, logical):
         """Add absent indentation for hanging indentation."""
         # Fix by adding an initial indent.
-        return self._fix_reindent(result, logical)
+        modified_lines = self._fix_reindent(result, logical)
+        if modified_lines:
+            return modified_lines
+        else:
+            # Fallback
+            line_index = result['line'] - 1
+            original_line = self.source[line_index]
+            indentation = _get_indentation(original_line)
+            self.source[line_index] = (indentation + self.indent_word +
+                                       original_line.lstrip())
 
     def fix_e123(self, result, logical):
         """Align closing bracket to match opening bracket."""
@@ -501,10 +525,7 @@ class FixPEP8(object):
                                offset=offset,
                                replacement='')
 
-        if fixed == target:
-            return []
-        else:
-            self.source[line_index] = fixed
+        self.source[line_index] = fixed
 
     def fix_e224(self, result):
         """Remove extraneous whitespace around operator."""
@@ -530,8 +551,10 @@ class FixPEP8(object):
     def fix_e231(self, result):
         """Add missing whitespace."""
         # Optimize for comma case. This will fix all commas in the full source
-        # code in one pass.
-        if ',' in result['info']:
+        # code in one pass. Don't do this more than once. If it fails the first
+        # time, there is no point in trying again.
+        if ',' in result['info'] and not self._ws_comma_done:
+            self._ws_comma_done = True
             original = ''.join(self.source)
             new = refactor(original, ['ws_comma'])
             if original.strip() != new.strip():
@@ -564,9 +587,7 @@ class FixPEP8(object):
         #
         #     def foo(a=\
         #             1)
-        if (fixed.endswith('=\\\n') or
-                fixed.endswith('=\\\r\n') or
-                fixed.endswith('=\\\r')):
+        if fixed.endswith(('=\\\n', '=\\\r\n', '=\\\r')):
             self.source[line_index] = fixed.rstrip('\n\r \t\\')
             self.source[line_index + 1] = self.source[line_index + 1].lstrip()
             return [line_index + 1, line_index + 2]  # Line indexed at 1
@@ -672,10 +693,10 @@ class FixPEP8(object):
         target = self.source[line_index]
 
         if target.lstrip().startswith('#'):
-            # Shorten comment if it is the last comment line.
+            last_comment = True
             try:
                 if self.source[line_index + 1].lstrip().startswith('#'):
-                    return []
+                    last_comment = False
             except IndexError:
                 pass
 
@@ -683,12 +704,10 @@ class FixPEP8(object):
             fixed = shorten_comment(
                 line=target,
                 newline=self.newline,
-                max_line_length=self.options.max_line_length)
-            if fixed == self.source[line_index]:
-                return []
-            else:
-                self.source[line_index] = fixed
-                return
+                max_line_length=self.options.max_line_length,
+                last_comment=last_comment)
+            self.source[line_index] = fixed
+            return
 
         indent = _get_indentation(target)
         source = target[len(indent):]
@@ -712,10 +731,12 @@ class FixPEP8(object):
         candidates = shorten_line(
             tokens, source, indent,
             self.indent_word, newline=self.newline,
-            aggressive=self.options.aggressive)
+            aggressive=self.options.aggressive,
+            previous_line=(
+                self.source[line_index - 1] if line_index >= 1 else ''))
 
         candidates = list(sorted(
-            set(candidates),
+            set(candidates).union([target]),
             key=lambda x: line_shortening_rank(x,
                                                self.newline,
                                                self.indent_word)))
@@ -727,17 +748,8 @@ class FixPEP8(object):
                                                             'buffer')
                                                  else sys.stderr))
 
-        for _candidate in candidates:
-            assert _candidate is not None
-
-            if (get_longest_length(_candidate, self.newline) >=
-                    get_longest_length(target, self.newline)):
-                continue
-
-            self.source[line_index] = _candidate
-            return
-
-        return []
+        if candidates:
+            self.source[line_index] = candidates[0]
 
     def fix_e502(self, result):
         """Remove extraneous escape of newline."""
@@ -840,7 +852,7 @@ class FixPEP8(object):
     def fix_w291(self, result):
         """Remove trailing whitespace."""
         fixed_line = self.source[result['line'] - 1].rstrip()
-        self.source[result['line'] - 1] = '%s%s' % (fixed_line, self.newline)
+        self.source[result['line'] - 1] = fixed_line + self.newline
 
     def fix_w293(self, result):
         """Remove trailing whitespace on blank line."""
@@ -868,13 +880,15 @@ def fix_e26(source):
         # Optimization.
         return source
 
-    string_line_numbers = multiline_string_lines(source,
-                                                 include_docstrings=True)
+    ignored_line_numbers = multiline_string_lines(
+        source,
+        include_docstrings=True) | set(commented_out_code_lines(source))
+
     fixed_lines = []
     sio = StringIO(source)
     for (line_number, line) in enumerate(sio.readlines(), start=1):
         if (line.lstrip().startswith('#') and
-                line_number not in string_line_numbers):
+                line_number not in ignored_line_numbers):
 
             indentation = _get_indentation(line)
             line = line.lstrip()
@@ -926,8 +940,6 @@ def fix_w6(source):
     return refactor(source,
                     ['apply',
                      'except',
-                     'exec',
-                     'execfile',
                      'exitfunc',
                      'has_key',
                      'idioms',
@@ -1028,8 +1040,6 @@ def _priority_key(pep8_result):
         'e225', 'e231',
         # Remove extraneous whitespace before breaking lines.
         'e201',
-        # Before breaking lines.
-        'e121', 'e122', 'e123', 'e124', 'e125', 'e126', 'e127', 'e128', 'e129',
     ]
     key = pep8_result['id'].lower()
     if key in priority:
@@ -1040,7 +1050,7 @@ def _priority_key(pep8_result):
 
 
 def shorten_line(tokens, source, indentation, indent_word, newline,
-                 aggressive=False):
+                 aggressive=False, previous_line=''):
     """Separate line at OPERATOR.
 
     Multiple candidates will be yielded.
@@ -1051,7 +1061,8 @@ def shorten_line(tokens, source, indentation, indent_word, newline,
                                    indentation=indentation,
                                    indent_word=indent_word,
                                    newline=newline,
-                                   aggressive=aggressive):
+                                   aggressive=aggressive,
+                                   previous_line=previous_line):
         yield candidate
 
     if aggressive:
@@ -1070,16 +1081,30 @@ def shorten_line(tokens, source, indentation, indent_word, newline,
 
 
 def _shorten_line(tokens, source, indentation, indent_word, newline,
-                  aggressive=False):
+                  aggressive=False, previous_line=''):
     """Separate line at OPERATOR.
 
     Multiple candidates will be yielded.
 
     """
     for tkn in tokens:
-        # Don't break on '=' after keyword as this violates PEP 8.
-        if token.OP == tkn[0] and tkn[1] != '=':
-            assert tkn[0] != token.INDENT
+        token_type = tkn[0]
+        token_string = tkn[1]
+
+        if (
+            token_type == tokenize.COMMENT and
+            not previous_line.rstrip().endswith('\\')
+        ):
+            # Move inline comments to previous line.
+            offset = tkn[2][1]
+            first = source[:offset]
+            second = source[offset:]
+            yield (indentation + second.strip() + newline +
+                   indentation + first.strip() + newline)
+        elif token_type == token.OP and token_string != '=':
+            # Don't break on '=' after keyword as this violates PEP 8.
+
+            assert token_type != token.INDENT
 
             offset = tkn[2][1] + 1
             first = source[:offset]
@@ -1102,7 +1127,7 @@ def _shorten_line(tokens, source, indentation, indent_word, newline,
             # Do end a line with a dot
             if first.rstrip().endswith('.'):
                 continue
-            if tkn[1] in '+-*/':
+            if token_string in '+-*/':
                 fixed = first + ' \\' + newline + second
             else:
                 fixed = first + newline + second
@@ -1507,7 +1532,7 @@ class Wrapper(object):
         last_indent = (0, 0)
         last_token_multiline = None
 
-        for token_type, text, start, end, _ in self.tokens:
+        for token_type, text, start, end, line in self.tokens:
             newline = row < start[0] - first_row
             if newline:
                 row = start[0] - first_row
@@ -1587,7 +1612,7 @@ class Wrapper(object):
                 # pep8.
                 visual_indent = indent_chances.get(start[1])
                 last_indent = start
-                rel_indent[row] = start[1] - indent_level
+                rel_indent[row] = pep8.expand_indent(line) - indent_level
                 hang = rel_indent[row] - rel_indent[open_row]
 
                 if token_type == tokenize.OP and text in ']})':
@@ -1653,7 +1678,12 @@ def refactor_with_2to3(source_text, fixer_names):
     from lib2to3.refactor import RefactoringTool
     fixers = ['lib2to3.fixes.fix_' + name for name in fixer_names]
     tool = RefactoringTool(fixer_names=fixers, explicit=fixers)
-    return unicode(tool.refactor_string(source_text, name=''))
+
+    from lib2to3.pgen2 import tokenize as lib2to3_tokenize
+    try:
+        return unicode(tool.refactor_string(source_text, name=''))
+    except lib2to3_tokenize.TokenError:
+        return source_text
 
 
 def break_multiline(source_text, newline, indent_word):
@@ -1667,11 +1697,11 @@ def break_multiline(source_text, newline, indent_word):
     # Handle special case only.
     for symbol in '([{':
         # Only valid if symbol is not on a line by itself.
-        if (symbol in source_text and not source_text.strip() == symbol):
-
-            if not source_text.rstrip()[-1] == ',':
-                continue
-
+        if (
+            symbol in source_text and
+            source_text.strip() != symbol and
+            source_text.rstrip().endswith((',', '%'))
+        ):
             index = 1 + source_text.find(symbol)
 
             if index <= len(indent_word) + len(indentation):
@@ -1723,6 +1753,8 @@ def filter_results(source, results, aggressive=False):
     all_string_line_numbers = multiline_string_lines(
         source, include_docstrings=True)
 
+    commented_out_code_line_numbers = commented_out_code_lines(source)
+
     split_source = [None] + source.splitlines()
 
     for r in results:
@@ -1738,6 +1770,14 @@ def filter_results(source, results, aggressive=False):
             if issue_id in ['e501']:
                 continue
 
+        # We must offset by 1 for lines that contain the trailing contents of
+        # multiline strings.
+        if not aggressive and (r['line'] + 1) in all_string_line_numbers:
+            # Do not modify multiline strings in non-aggressive mode. Remove
+            # trailing whitespace could break doctests.
+            if issue_id.startswith('w29') or issue_id.startswith('w39'):
+                continue
+
         # Filter out incorrect E101 reports when there are no tabs.
         # pep8 will complain about this even if the tab indentation found
         # elsewhere is in a multiline string.
@@ -1747,6 +1787,10 @@ def filter_results(source, results, aggressive=False):
         if issue_id in ['e711', 'e712'] and not aggressive:
             continue
 
+        if r['line'] in commented_out_code_line_numbers:
+            if issue_id == 'e501' or issue_id.startswith('e26'):
+                continue
+
         yield r
 
 
@@ -1766,12 +1810,12 @@ def multiline_string_lines(source, include_docstrings=False):
             token_type = t[0]
             start_row = t[2][0]
             end_row = t[3][0]
-            start_row = t[2][0]
-            end_row = t[3][0]
 
             if (token_type == tokenize.STRING and start_row != end_row):
-                if (include_docstrings or
-                        previous_token_type != tokenize.INDENT):
+                if (
+                    include_docstrings or
+                    previous_token_type != tokenize.INDENT
+                ):
                     # We increment by one since we want the contents of the
                     # string.
                     line_numbers |= set(range(1 + start_row, 1 + end_row))
@@ -1783,7 +1827,41 @@ def multiline_string_lines(source, include_docstrings=False):
     return line_numbers
 
 
-def shorten_comment(line, newline, max_line_length):
+def commented_out_code_lines(source):
+    """Return line numbers of comments that are likely code.
+
+    Commented-out code is bad practice, but modifying it just adds even more
+    clutter.
+
+    """
+    sio = StringIO(source)
+    line_numbers = []
+    try:
+        for t in tokenize.generate_tokens(sio.readline):
+            token_type = t[0]
+            token_string = t[1]
+            start_row = t[2][0]
+            line = t[4]
+
+            # Ignore inline comments.
+            if not line.lstrip().startswith('#'):
+                continue
+
+            if token_type == tokenize.COMMENT:
+                stripped_line = token_string.lstrip('#').strip()
+                if (
+                    ' ' in stripped_line and
+                    '#' not in stripped_line and
+                    check_syntax(stripped_line)
+                ):
+                    line_numbers.append(start_row)
+    except (SyntaxError, tokenize.TokenError):
+        pass
+
+    return line_numbers
+
+
+def shorten_comment(line, newline, max_line_length, last_comment=False):
     """Return trimmed or split long comment line."""
     assert len(line) > max_line_length
     line = line.rstrip()
@@ -1798,7 +1876,7 @@ def shorten_comment(line, newline, max_line_length):
             not line[-1].isalnum()):
         # Trim comments that end with things like ---------
         return line[:max_line_length] + newline
-    elif re.match(r'\s*#+\s*\w+', line):
+    elif last_comment and re.match(r'\s*#+\s*\w+', line):
         import textwrap
         split_lines = textwrap.wrap(line.lstrip(' \t#'),
                                     initial_indent=indentation,
@@ -1936,7 +2014,7 @@ def global_fixes():
 def apply_global_fixes(source, options):
     """Run global fixes on source code.
 
-    Thsese are fixes that only need be done once (unlike those in FixPEP8,
+    These are fixes that only need be done once (unlike those in FixPEP8,
     which are dependent on pep8).
 
     """
@@ -1969,7 +2047,7 @@ def extract_code_from_function(function):
 
 def parse_args(args):
     """Parse command-line options."""
-    parser = OptionParser(usage='Usage: autopep8 [options] '
+    parser = OptionParser(usage='Usage: %prog [options] '
                                 '[filename [filename ...]]'
                                 '\nUse filename \'-\'  for stdin.',
                           version='%prog {0}'.format(__version__),
@@ -2121,11 +2199,10 @@ def line_shortening_rank(candidate, newline, indent_word):
             if (bad_staring_symbol and
                     lines[1].lstrip().startswith(bad_staring_symbol)):
                 rank += 20
-            else:
-                rank -= 10
 
-        if lines[0].endswith('(['):
-            rank += 10
+        if re.match(r'.*[+\-\*/] \($', lines[0]):
+            # "1 * (\n" is ugly as hell.
+            rank += 100
 
         for current_line in lines:
             for bad_start in ['.', '%', '+', '-', '/']:
@@ -2181,11 +2258,6 @@ def split_at_offsets(line, offsets):
     return result
 
 
-def get_longest_length(text, newline):
-    """Return length of longest line."""
-    return max([len(line) for line in text.split(newline)])
-
-
 class LineEndingWrapper(object):
 
     r"""Replace line endings to work with sys.stdout.
@@ -2272,10 +2344,6 @@ def is_python_file(filename):
     except (IOError, IndexError):
         return False
 
-    if len(first_line) > 200:
-        # This is probably not even a text file.
-        return False
-
     if not PYTHON_SHEBANG_REGEX.match(first_line):
         return False
 
